=== modified file 'lib/lp/soyuz/scripts/gina/dominate.py'
--- lib/lp/soyuz/scripts/gina/dominate.py	2011-09-16 11:41:14 +0000
+++ lib/lp/soyuz/scripts/gina/dominate.py	2011-09-19 04:26:46 +0000
@@ -20,86 +20,14 @@
     series = getUtility(IDistributionSet)[distro_name].getSeries(series_name)
     dominator = Dominator(logger, series.main_archive)
 
-    # XXX JeroenVermeulen 2011-09-08, bug=844550: This is a transitional
-    # hack.  Gina used to create SPPHs in Pending state.  We cleaned up
-    # the bulk of them, and changed the code to create Published ones, but
-    # some new ones will have been created since.
-    # Update those to match what the new Gina does.
-    from canonical.launchpad.interfaces.lpstorm import IStore
-    from lp.soyuz.enums import PackagePublishingStatus
-    from lp.soyuz.model.publishing import SourcePackagePublishingHistory
-    SPPH = SourcePackagePublishingHistory
-    store = IStore(SPPH)
-    spphs = store.find(
-        SPPH,
-        SPPH.archive == series.main_archive,
-        SPPH.distroseries == series,
-        SPPH.pocket == pocket,
-        SPPH.status == PackagePublishingStatus.PENDING)
-    spphs.set(status=PackagePublishingStatus.PUBLISHED)
-
-    # Dominate packages found in the Sources list we're importing.
+    # Dominate all packages published in the series.  This includes all
+    # packages listed in the Sources file we imported, but also packages
+    # that have been recently deleted.
     package_names = dominator.findPublishedSourcePackageNames(series, pocket)
     for package_name in package_names:
-        entries = packages_map.src_map.get(package_name)
-
-        if entries is None:
-            # XXX JeroenVermeulen 2011-09-08, bug=844550: This is a
-            # transitional hack.  The database is full of "Published"
-            # Debian SPPHs whose packages have actually been deleted.
-            # In the future such publications should simply be marked
-            # Deleted, but for the legacy baggage we currently carry
-            # around we'll just do traditional domination first: pick
-            # the latest Published version, and mark the rest of the
-            # SPPHs as superseded by that version.  The latest version
-            # will then, finally, be marked appropriately Deleted once
-            # we remove this transitional hack.
-            # To remove the transitional hack, just let live_versions
-            # default to the empty list instead of doing this:
-            import apt_pkg
-            from lp.services.database.bulk import load_related
-            from lp.soyuz.model.sourcepackagerelease import (
-                SourcePackageRelease,
-                )
-            pubs = list(
-                dominator.findPublishedSPPHs(series, pocket, package_name))
-            if len(pubs) <= 1:
-                # Without at least two published SPPHs, the transitional
-                # code will make no changes.  Skip, and leave the
-                # algorithm free to assume there's a pubs[0].
-                continue
-            load_related(
-                SourcePackageRelease, pubs, ['sourcepackagereleaseID'])
-
-            # Close off the transaction to avoid being idle-killed.
-            # Nothing else is going to supersede or delete these
-            # publications in the meantime, so our data stays valid; and
-            # if new ones become published, they still won't be
-            # considered anyway.
-            txn.commit()
-
-            # Find the "latest" publication.  A purely in-memory
-            # operation; won't open a new transaction.
-            def is_newer(candidate, reference):
-                comparison = apt_pkg.VersionCompare(
-                    candidate.sourcepackagerelease.version,
-                    reference.sourcepackagerelease.version)
-                if comparison > 0:
-                    return True
-                elif comparison < 0:
-                    return False
-                else:
-                    return candidate.datecreated > reference.datecreated
-
-            latest_pub = pubs[0]
-            for pub in pubs[1:]:
-                if is_newer(pub, latest_pub):
-                    latest_pub = pub
-            live_versions = [latest_pub.sourcepackagerelease.version]
-        else:
-            live_versions = [
-                entry['Version']
-                for entry in entries if 'Version' in entry]
+        entries = packages_map.src_map.get(package_name, [])
+        live_versions = [
+            entry['Version'] for entry in entries if 'Version' in entry]
 
         dominator.dominateRemovedSourceVersions(
             series, pocket, package_name, live_versions)

=== modified file 'lib/lp/soyuz/scripts/tests/test_gina.py'
--- lib/lp/soyuz/scripts/tests/test_gina.py	2011-09-16 10:48:59 +0000
+++ lib/lp/soyuz/scripts/tests/test_gina.py	2011-09-19 10:05:18 +0000
@@ -36,15 +36,51 @@
         # packages that Gina imports.
         logger = DevNullLogger()
         txn = FakeTransaction()
-        pub = self.factory.makeSourcePackagePublishingHistory(
-            status=PackagePublishingStatus.PUBLISHED)
-        series = pub.distroseries
-        spr = pub.sourcepackagerelease
-        package = spr.sourcepackagename
+        series = self.factory.makeDistroSeries()
+        pocket = PackagePublishingPocket.RELEASE
+        package = self.factory.makeSourcePackageName()
+
+        # Realistic situation: there's an older, superseded publication;
+        # a series of active ones; and a newer, pending publication
+        # that's not in the Sources lists yet.
+        # Gina dominates the Published ones and leaves the rest alone.
+        old_spph = self.factory.makeSourcePackagePublishingHistory(
+            distroseries=series, archive=series.main_archive,
+            pocket=pocket, status=PackagePublishingStatus.SUPERSEDED,
+            sourcepackagerelease=self.factory.makeSourcePackageRelease(
+                sourcepackagename=package, version='1.0'))
+
+        active_spphs = [
+            self.factory.makeSourcePackagePublishingHistory(
+                distroseries=series, archive=series.main_archive,
+                pocket=pocket, status=PackagePublishingStatus.PUBLISHED,
+                sourcepackagerelease=self.factory.makeSourcePackageRelease(
+                    sourcepackagename=package, version=version))
+            for version in ['1.1', '1.1.1', '1.1.1.1']]
+
+        new_spph = self.factory.makeSourcePackagePublishingHistory(
+            distroseries=series, archive=series.main_archive,
+            pocket=pocket, status=PackagePublishingStatus.PENDING,
+            sourcepackagerelease=self.factory.makeSourcePackageRelease(
+                sourcepackagename=package, version='1.2'))
+
+        spphs = [old_spph] + active_spphs + [new_spph]
+
+        # Of the active publications, in this scenario, only one version
+        # matches what Gina finds in the Sources list.  It stays
+        # published; older active publications are superseded, newer
+        # ones deleted.
         dominate_imported_source_packages(
-            txn, logger, series.distribution.name, series.name, pub.pocket,
-            FakePackagesMap({package.name: []}))
-        self.assertEqual(PackagePublishingStatus.DELETED, pub.status)
+            txn, logger, series.distribution.name, series.name, pocket,
+            FakePackagesMap({package.name: [{'Version': '1.1.1'}]}))
+        self.assertEqual([
+            PackagePublishingStatus.SUPERSEDED,
+            PackagePublishingStatus.SUPERSEDED,
+            PackagePublishingStatus.PUBLISHED,
+            PackagePublishingStatus.DELETED,
+            PackagePublishingStatus.PENDING,
+            ],
+            [pub.status for pub in spphs])
 
     def test_dominate_imported_source_packages_dominates_deletions(self):
         # dominate_imported_source_packages dominates the source
@@ -60,81 +96,25 @@
                 sourcepackagerelease=self.factory.makeSourcePackageRelease(
                     sourcepackagename=package, version=version))
             for version in ['1.0', '1.1', '1.1a']]
+
+        # In this scenario, 1.0 is a superseded release.
+        pubs[0].supersede()
         logger = DevNullLogger()
         txn = FakeTransaction()
         dominate_imported_source_packages(
             txn, logger, series.distribution.name, series.name, pocket,
             FakePackagesMap({}))
-        # XXX JeroenVermeulen 2011-09-08, bug=844550: This is
-        # "transitional" domination which supersedes older versions of
-        # deleted packages with the last known version.  Permanent
-        # domination will then mark the last known version as deleted.
-        # For permanent domination, the expected outcome is that all
-        # these publications will be Deleted (but any pre-existing
-        # Superseded publications for older versions will remain
-        # Superseded).
+
+        # The older, superseded release stays superseded; but the
+        # releases that dropped out of the imported Sources list without
+        # known successors are marked deleted.
         self.assertEqual([
             PackagePublishingStatus.SUPERSEDED,
-            PackagePublishingStatus.SUPERSEDED,
-            PackagePublishingStatus.PUBLISHED,
+            PackagePublishingStatus.DELETED,
+            PackagePublishingStatus.DELETED,
             ],
             [pub.status for pub in pubs])
 
-    def test_dominate_imported_source_packages_cleans_up_pending_spphs(self):
-        # XXX JeroenVermeulen 2011-09-08, bug=844550: For transition to
-        # Gina domination, dominate_imported_source_packages turns any
-        # remaining Pending SPPHS into Published ones.
-        series = self.factory.makeDistroSeries()
-        spph = self.factory.makeSourcePackagePublishingHistory(
-            distroseries=series, archive=series.main_archive,
-            status=PackagePublishingStatus.PENDING)
-        spr = spph.sourcepackagerelease
-        package_name = spr.sourcepackagename.name
-        logger = DevNullLogger()
-        txn = FakeTransaction()
-        dominate_imported_source_packages(
-            txn, logger, series.distribution.name, series.name, spph.pocket,
-            FakePackagesMap({package_name: [{"Version": spr.version}]}))
-        self.assertEqual(PackagePublishingStatus.PUBLISHED, spph.status)
-
-    def test_dominate_imported_source_packages_cleans_up_first(self):
-        # XXX JeroenVermeulen 2011-09-08, bug=844550: For transition to
-        # Gina domination, dominate_imported_source_packages turns any
-        # remaining Pending SPPHS into Published ones.  It does this
-        # *before* dominating, so no domination happens while some of
-        # the SPPHs are still mistakenly Pending (which would result in
-        # mistaken deletions).
-        series = self.factory.makeDistroSeries()
-        package = self.factory.makeSourcePackageName()
-        pocket = PackagePublishingPocket.RELEASE
-        versions = ['1.0', '1.1']
-        statuses_before = [
-            PackagePublishingStatus.PUBLISHED,
-            PackagePublishingStatus.PENDING,
-            ]
-        statuses_after = [
-            PackagePublishingStatus.SUPERSEDED,
-            PackagePublishingStatus.PUBLISHED,
-            ]
-        live_version = versions[-1]
-        sprs = [
-            self.factory.makeSourcePackageRelease(
-                sourcepackagename=package, version=version)
-            for version in versions]
-        spphs = [
-            self.factory.makeSourcePackagePublishingHistory(
-                archive=series.main_archive, distroseries=series,
-                sourcepackagerelease=spr, pocket=pocket, status=status)
-            for spr, status in zip(sprs, statuses_before)]
-
-        logger = DevNullLogger()
-        txn = FakeTransaction()
-        dominate_imported_source_packages(
-            txn, logger, series.distribution.name, series.name, pocket,
-            FakePackagesMap({package.name: [{"Version": live_version}]}))
-
-        self.assertEqual(statuses_after, [spph.status for spph in spphs])
-
 
 class TestSourcePackagePublisher(TestCaseWithFactory):
 

